@@ -9,28 +9,30 @@ use crate::primes::PRIMELIST;
 impl NumberTheory for Mpz{
   
   
-  fn rng() -> Self {Mpz::new(Sign::Positive, vec![rng_64(), rng_64(), rng_64(), rng_64()])}
+  fn rng() -> Self {Mpz::unchecked_new(Sign::Positive, vec![rng_64(), rng_64(), rng_64(), rng_64()])}
   
-  fn euclidean(&self, other:&Self) -> (Self,Self) {
-     if self.sign==Sign::Negative || other.sign == Sign::Negative {
-        let (quo, mut rem) = self.ref_euclidean(other);
+  
+  
+  
+  fn euclidean_div(&self, other:&Self) -> (Self,Self) {
+        let (mut quo, mut rem) = self.ref_euclidean(other);
+     if self.sign==Sign::Negative && other.sign == Sign::Negative {
         rem.neg();
-        return (quo,rem)
      }
-     else if self.sign==Sign::Positive || other.sign == Sign::Negative {
-       let (mut quo, mut rem) = self.ref_euclidean(other);
+     else if self.sign==Sign::Positive && other.sign == Sign::Negative {
         quo.neg();
-        return (quo,rem)
      }
-     else if self.sign==Sign::Negative || other.sign == Sign::Positive{
-       let (mut quo, mut rem) = self.ref_euclidean(other);
+     else if self.sign==Sign::Negative && other.sign == Sign::Positive{
         quo.neg();
         rem.neg();
-        return (quo,rem)
      }
-     self.ref_euclidean(other)
+     
+    quo.fix_zero();
+    rem.fix_zero();
+    (quo,rem)
   }
   
+  
   fn quadratic_residue(&self, n: &Self) -> Self{
   
      if n == &Mpz::zero(){
@@ -90,7 +92,7 @@ impl NumberTheory for Mpz{
  
  }
  
- fn gcd(&self, other: &Self)->Self{
+ fn euclid_gcd(&self, other: &Self)->Self{
          let mut a = self.clone();
          let mut b = other.clone();
 
@@ -126,7 +128,7 @@ impl NumberTheory for Mpz{
         
          
           if rem == 0{
-        //  println!("rem hit {}", i);
+
            let mut count = 1u64;
              factors.push(Mpz::from_u64(*i as u64));
              n = quo;
@@ -149,7 +151,7 @@ impl NumberTheory for Mpz{
           
           }
           n.normalize();
-       // println!("{:?}", n);
+
           if n == Mpz::one() {
             return factors
           }
@@ -159,7 +161,7 @@ impl NumberTheory for Mpz{
              factors.push(Mpz::one());
              return factors
           }
-         // println!("next step{}", n.to_string());
+
        'outer : while n != Mpz::one(){
           
               let k = n.rho_mpz();
@@ -234,6 +236,7 @@ impl NumberTheory for Mpz{
  
  fn legendre(&self, p: &Self) -> i8 {
      let mut p_minus = p.clone();
+     p_minus.set_sign(Sign::Positive);
      sub_slice(&mut p_minus.limbs[..], &[1]);
      let pow = p_minus.ref_euclidean(&Mpz::from_u64(2)).0;
      let k = self.mod_pow(&pow,&p);
@@ -243,11 +246,59 @@ impl NumberTheory for Mpz{
  }
  
  fn checked_legendre(&self, p: &Self) -> Option<i8> {
-      if p == &Mpz::from_u64(2) {return None}
-     match p.is_prime(){
-       true  => Some(self.legendre(p)),
+     let mut plus = p.clone();
+     plus.set_sign(Sign::Positive);
+      if plus == Mpz::from_u64(2) {return None}
+     match plus.is_prime(){
+       true  => Some(self.legendre(&plus)),
        false => None,
      }
  }
+ 
+ 
+ fn jacobi(&self, k: &Self) -> i8 {
+    let mut n = self.clone();
+    let mut p = k.clone();
+    let mut t = 1i8;
+    n = n.euclidean_div(&p).1;
+
+    while n != Mpz::zero(){
+     let zeros = n.trailing_zeros(); 
+     n.mut_shr(zeros as usize);
+     
+     if (p.congruence_u64(8,3) || p.congruence_u64(8,5)) && (zeros%2 == 1) { 
+
+            t = -t
+     }
+    
+        std::mem::swap(&mut n, &mut p);
+        
+        if n.congruence_u64(4,3) && p.congruence_u64(4,3) {
+
+            t = -t;
+        }
+
+        n = n.euclidean_div(&p).1;
+
+       
+    }
+    
+    if p == Mpz::one() {
+        t
+    } 
+    
+    else {
+        0
+    }
+}
+
+fn checked_jacobi(&self, k: &Self) -> Option<i8>{
+    if k.sign == Sign::Positive && k != &Mpz::zero() && k.is_even() == false {
+      return Some(self.jacobi(k))
+    }
+     return None
+ }
+ 
+ 
   
   }
