@@ -9,7 +9,7 @@ impl NumberTheory for u32{
 
  fn rng() -> Self {rng_32() }
 
- fn euclidean(&self, other: &Self) -> (Self,Self) {
+ fn euclidean_div(&self, other: &Self) -> (Self,Self) {
    (*self/ *other, *self%*other)
   }
 
@@ -91,7 +91,7 @@ impl NumberTheory for u32{
    }
  
  
- fn gcd(&self, other: &Self) -> Self{
+ fn euclid_gcd(&self, other: &Self) -> Self{
      let mut a = self.clone();
      let mut b = other.clone();
      if b == 0 
@@ -174,12 +174,50 @@ impl NumberTheory for u32{
  }
  
  fn checked_legendre(&self, p: &Self) -> Option<i8> {
-     if *p == 2 {return None}
-     match p.is_prime(){
-       true  => Some(self.legendre(p)),
-       false => None,
-     }
+    if p == &2 || p.is_prime() == false {
+        return None
+      } 
+     Some(self.legendre(&p))
  }
+ 
+ fn jacobi(&self, k: &Self) -> i8 {
+    let mut n = *self;
+    let mut p = *k;
+    let mut t = 1i8;
+    n %= p;
+    
+    while n != 0 {
+     let zeros = n.trailing_zeros(); 
+     n>>=zeros;
+     
+     if (p % 8 == 3 || p % 8 == 5) && (zeros%2 == 1) { 
+            t = -t
+     }
+    
+        std::mem::swap(&mut n, &mut p);
+        if n % 4 == 3 && p % 4 == 3 {
+            t = -t;
+        }
+        n %= p;
+    }
+    
+    if p == 1 {
+        t
+    } 
+    
+    else {
+        0
+    }
+}
+
+fn checked_jacobi(&self, k: &Self) -> Option<i8>{
+    if k > &0 && *k % 2 == 1 {
+     return Some(self.jacobi(k))
+    }
+     return None
+ }
+ 
+ 
 
 }
 
@@ -189,7 +227,7 @@ impl NumberTheory for i32{
   
   fn rng() -> Self {rng_32() as i32}
   
-  fn euclidean(&self, other: &Self) -> (Self,Self) {
+  fn euclidean_div(&self, other: &Self) -> (Self,Self) {
    (*self/ *other, *self%*other)
   }
   
@@ -209,8 +247,8 @@ impl NumberTheory for i32{
       (self.abs() as u32).k_free(&(k.abs() as u32))
   }
   
-  fn gcd(&self, other: &Self) -> Self{
-      (self.abs() as u32).gcd(&(other.abs() as u32)) as i32
+  fn euclid_gcd(&self, other: &Self) -> Self{
+      (self.abs() as u32).euclid_gcd(&(other.abs() as u32)) as i32
   }
   
   fn euler_totient(&self) -> Self{
@@ -244,12 +282,59 @@ impl NumberTheory for i32{
   }
   
   fn legendre(&self, p: &Self) -> i8 {
-       (self.abs() as u32).legendre(&(p.abs() as u32))
+    let k = self.mod_pow(&((p.abs()-1)>>1), &p.abs());
+    if k == 1{return 1};
+    if k == p.abs()-1 {return -1};
+    return 0
  }
  
   fn checked_legendre(&self, p: &Self) -> Option<i8> {
-     (self.abs() as u32).checked_legendre(&(p.abs() as u32))
+     if p.abs() == 2 || p.is_prime() == false {
+        return None
+      } 
+     Some(self.legendre(&p))
+     
  }
+ 
+ 
+ fn jacobi(&self, k: &Self) -> i8 {
+    let mut n = *self;
+    let mut p = *k;
+    let mut t = 1i8;
+    n %= p;
+    
+    while n != 0 {
+     let zeros = n.trailing_zeros(); 
+     n>>=zeros;
+     
+     if (p % 8 == 3 || p % 8 == 5) && (zeros%2 == 1) { 
+            t = -t
+     }
+    
+        std::mem::swap(&mut n, &mut p);
+        if n % 4 == 3 && p % 4 == 3 {
+            t = -t;
+        }
+        n %= p;
+    }
+    
+    if p == 1 {
+        t
+    } 
+    
+    else {
+        0
+    }
+}
+
+fn checked_jacobi(&self, k: &Self) -> Option<i8>{
+    if k > &0 && *k % 2 == 1 {
+      return Some(self.jacobi(k))
+    }
+     return None
+ }
+ 
+ 
 }
 
   // 32-bit pollard rho
@@ -267,7 +352,7 @@ impl NumberTheory for i32{
     ((x as u64 * x as u64 + 1 )%n as u64) as u32
    }
  
-  // 64-bit pollard rho
+  // 32-bit pollard rho
   fn rho_32(n: u32)->u32{
 
   let mut x = 2; let mut y = 2; let mut d = 1;
@@ -275,7 +360,7 @@ impl NumberTheory for i32{
   while d == 1 {
   x = mod_sqr1_32(x,n);
   y = mod_sqr1_32(mod_sqr1_32(y,n),n)%n;
-  d = delta_u32(x,y).gcd(&n)
+  d = delta_u32(x,y).euclid_gcd(&n)
    }
    d
 }
