@@ -3,7 +3,7 @@ List of first 2048 primes
 
 */
 
- pub(crate) const PRIMELIST :  [u16;2048] =  [
+ pub/*(crate)*/ const PRIMELIST :  [u16;2048] =  [
 
   
    2, 3, 5, 7, 11, 13, 17, 19, 23, 29, 31, 37, 41, 43, 47, 53, 59, 61, 67, 71,
