@@ -8,7 +8,7 @@ pub trait NumberTheory{
    ///Random number generation, generally high-quality
    fn rng() -> Self;
    /// Euclidean function, normally called euclidean division
-   fn euclidean(&self, other: &Self) -> (Self,Self) where Self: Sized;
+   fn euclidean_div(&self, other: &Self) -> (Self,Self) where Self: Sized;
    ///Returns optimally fast primality check for each interval. 
    /// Deterministic for n < 2^64. For n > 2^64 then the probability of failure is > 2^-64. Unlike most libraries Number-Theory weights the number of Strong Fermat checks
    /// to guarantee a constant error rate, resulting in being slightly slower, but more accurate, for small numbers but faster for larger numbers. Values greater than 2^512
@@ -19,7 +19,7 @@ pub trait NumberTheory{
    /// Factorizes into a vector of the form factor, power, factor, power . . . i.e 2,6,5,2 for 2^6 * 5^2  = 1600
    fn factor(&self)-> Vec<Self> where Self: Sized;
    /// Binary gcd, Computes the greatest common divisor of two numbers
-   fn gcd(&self, other: &Self) -> Self;
+   fn euclid_gcd(&self, other: &Self) -> Self;
    /// Counts the number of coprimes from 0 to self
    fn euler_totient(&self) -> Self;
      /// Computes self^p mod n
@@ -39,11 +39,19 @@ pub trait NumberTheory{
    /// Legendre symbol of a,p. Verifies that p is an odd prime, returns None if not. 
    fn checked_legendre(&self, p: &Self) -> Option<i8> ;
    
+   
+   /// Jacobi symbol of self,p. Assumes that p is an odd prime
+   fn jacobi(&self, p: &Self) -> i8 ;  
+   /// Jacobi symbol of self,p. Verifies that p is an odd prime, returns None if not. 
+   fn checked_jacobi(&self, p: &Self) -> Option<i8> ;
    /*
    
    /// Pi approximation
    fn pi(&self) -> f64;
    
+   let x = self.ln();
+    //(self/x )*(1.0 + 1.0/x  + 2.0/(x.ln()*x.ln()))
+       
    /// Smoothness bound of number
    fn smooth
    /// Checks if the smoothness bound is at least b
@@ -61,14 +69,21 @@ pub trait NumberTheory{
    fn jordan_totient
    
    
-   /// Jacobi symbol of a,p. Assumes that p is an odd prime
-   fn jacobi()   
-   /// Jacobi symbol of a,p. Verifies that p is an odd prime, returns None if not. 
-   fn checked_jacobi
    
     /// Integer partition
    fn partition()
    */
 }
 
+/*
+NTT research 
+
+  https://stackoverflow.com/questions/18465326/fast-bignum-square-computation
+  
+  https://stackoverflow.com/questions/18577076/modular-arithmetics-and-ntt-finite-field-dft-optimizations
+  https://stackoverflow.com/questions/10260927/translation-from-complex-fft-to-finite-field-fft/18547575#18547575
+  
+  https://dl.acm.org/doi/10.1145/1185448.1185568
+
+*/
 
