@@ -8,7 +8,7 @@ impl NumberTheory for u16{
 
  fn rng() -> Self {(rng_32()>>16) as u16}
  
- fn euclidean(&self, other: &Self) -> (Self,Self) {
+ fn euclidean_div(&self, other: &Self) -> (Self,Self) {
    (*self/ *other, *self%*other)
   }
 
@@ -75,7 +75,7 @@ impl NumberTheory for u16{
    }
  
  
- fn gcd(&self, other: &Self) -> Self{
+ fn euclid_gcd(&self, other: &Self) -> Self{
      let mut a = self.clone();
      let mut b = other.clone();
      if b == 0 
@@ -158,11 +158,47 @@ impl NumberTheory for u16{
  }
  
  fn checked_legendre(&self, p: &Self) -> Option<i8> {
-     if *p == 2 {return None}
-     match p.is_prime(){
-       true  => Some(self.legendre(p)),
-       false => None,
+     if p == &2 || p.is_prime() == false {
+          return None
+        } 
+       Some(self.legendre(&p))
+ }
+
+ fn jacobi(&self, k: &Self) -> i8 {
+    let mut n = *self;
+    let mut p = *k;
+    let mut t = 1i8;
+    n %= p;
+    
+    while n != 0 {
+     let zeros = n.trailing_zeros(); 
+     n>>=zeros;
+     
+     if (p % 8 == 3 || p % 8 == 5) && (zeros%2 == 1) { 
+            t = -t
      }
+    
+        std::mem::swap(&mut n, &mut p);
+        if n % 4 == 3 && p % 4 == 3 {
+            t = -t;
+        }
+        n %= p;
+    }
+    
+    if p == 1 {
+        t
+    } 
+    
+    else {
+        0
+    }
+}
+
+fn checked_jacobi(&self, k: &Self) -> Option<i8>{
+    if k > &0 && *k % 2 == 1 {
+    return  Some(self.jacobi(k))
+    }
+     return None
  }
 
 }
@@ -173,7 +209,7 @@ impl NumberTheory for i16{
   
   fn rng() -> Self {(rng_32()>>16) as i16}
   
-  fn euclidean(&self, other: &Self) -> (Self,Self) {
+  fn euclidean_div(&self, other: &Self) -> (Self,Self) {
    (*self/ *other, *self%*other)
   }
   
@@ -193,8 +229,8 @@ impl NumberTheory for i16{
       (self.abs() as u16).k_free(&(k.abs() as u16))
   }
   
-  fn gcd(&self, other: &Self) -> Self{
-      (self.abs() as u16).gcd( &(other.abs() as u16)) as i16
+  fn euclid_gcd(&self, other: &Self) -> Self{
+      (self.abs() as u16).euclid_gcd( &(other.abs() as u16)) as i16
   }
   
   fn euler_totient(&self) -> Self{
@@ -229,10 +265,54 @@ impl NumberTheory for i16{
   
   
   fn legendre(&self, p: &Self) -> i8 {
-       (self.abs() as u16).legendre(&(p.abs() as u16))
+   let k = self.mod_pow(&((p.abs()-1)>>1), &p.abs());
+    if k == 1{return 1};
+    if k == p.abs()-1 {return -1};
+    return 0
  }
  
   fn checked_legendre(&self, p: &Self) -> Option<i8> {
-     (self.abs() as u16).checked_legendre(&(p.abs() as u16))
+     if p.abs() == 2 || p.is_prime() == false {
+        return None
+      } 
+     Some(self.legendre(&p))
  }
+ 
+ fn jacobi(&self, k: &Self) -> i8 {
+    let mut n = *self;
+    let mut p = *k;
+    let mut t = 1i8;
+    n %= p;
+    
+    while n != 0 {
+     let zeros = n.trailing_zeros(); 
+     n>>=zeros;
+     
+     if (p % 8 == 3 || p % 8 == 5) && (zeros%2 == 1) { 
+            t = -t
+     }
+    
+        std::mem::swap(&mut n, &mut p);
+        if n % 4 == 3 && p % 4 == 3 {
+            t = -t;
+        }
+        n %= p;
+    }
+    
+    if p == 1 {
+        t
+    } 
+    
+    else {
+        0
+    }
+}
+
+fn checked_jacobi(&self, k: &Self) -> Option<i8>{
+    if k > &0 && *k % 2 == 1 {
+     return Some(self.jacobi(k))
+    }
+     return None
+ }
+ 
 }
